module AppPush
  module Huawei
    class << self
      attr_accessor :config, :access_token, :expire_time

      def config
        @config = {} if @config.blank?
        @config.merge!(default_config)
      end

      def default_config
        {
          grant_type: 'client_credentials'
        }
      end

      def push(device_tokens, message)
        device_tokens = Array(device_tokens) unless device_tokens.instance_of? Array
        handle_token_res if access_token.blank? || (expire_time.present? && Time.now >= expire_time)
        device_tokens.each_slice(100) do |device_token_array|
          res_push(message, device_token_array)
        end
      rescue StandardError => err
        AppPush::Log.error(err)
      end

      def res_push(message, device_tokens)
        body = message(message, device_tokens)
        headers = {
          'Content-Type' => 'application/x-www-form-urlencoded'
        }
        nsp_ctx = {
          ver: '1',
          appId: config['app_id']
        }
        url = "https://api.push.hicloud.com/pushsend.do?nsp_ctx=#{URI.escape(nsp_ctx.to_s)}"
        AppPush::Log.info("url: #{url}")
        AppPush::Log.info("body: #{body}")
        response = HTTParty.post(url, body: body, headers: headers, timeout: 5)
        AppPush::Log.info("response: #{response}")
        response
      end

      # 推送信息
      def message(message, device_tokens)
        payload = {
          hps: {
            msg: {
              type: 1,
              body: message
            }
          }
        }
        {
          access_token: 'access_token',
          nsp_svc: 'openpush.message.api.send',
          nsp_ts: Time.now.to_i,
          device_token_list: device_tokens, # 单次最多100
          payload: payload
        }
      end

      def handle_token_res
        loop do
          res = req_token
          if res['error'].blank?
            @access_token = res['access_token']
            break if @access_token.present?
          end
        end
        @expire_time = Time.now + 58.minutes
      end

      def req_token
        url = 'https://login.cloud.huawei.com/oauth2/v2/token'
        body = {
          grant_type: 'client_credentials',
          client_id: config['app_id'],
          client_secret: config['app_secret']
        }
        headers = {
          'Content-Type' => 'application/x-www-form-urlencoded'
        }
        AppPush::Log.info("url: #{url}")
        AppPush::Log.info("body: #{body}")
        response = HTTParty.post(url, body: body, headers: headers, timeout: 5)
        AppPush::Log.info("response: #{response}")
        JSON.parse(response)
      end
    end
  end
end
