module AppPush
  module Igetui
    class << self
      attr_accessor :config

      def push(user_device_ids, template_data = {})
        if user_device_ids.is_a? Array
          push_transmission_to_list_igetui(user_device_ids, template_data)
        else
          push_transmission_to_igetui(user_device_ids, template_data)
        end
      rescue StandardError => err
        AppPush::Log.error(err)
      end

      def config
        (@config || default_config).with_indifferent_access
      end

      def default_config
        {
          push_type: 2
        }
      end

      def push_transmission_to_igetui(user_device_id, template_data = {})
        template_data = handle_template_data(template_data)

        push_igetui_with_asnyc(
          user_device_id,
          template_data,
          template_type: 'TransmissionTemplate'
        )
      end

      def push_transmission_to_list_igetui(user_device_ids, template_data = {})
        template_data = handle_template_data(template_data)
        push_list_igetui_with_asnyc(
          user_device_ids,
          template_data,
          template_type: 'TransmissionTemplate', pusher_type: 'push_message_to_list'
        )
      end

      private

      def handle_template_data(template_data)
        template_data = HashWithIndifferentAccess.new(template_data)
        transmission_content = HashWithIndifferentAccess.new(template_data.delete(:transmission_content))
        transmission_content[:push_type] ||= config[:push_type]
        template_data[:transmission_type] ||= 0
        template_data[:transmission_content] = transmission_content.as_json
        template_data
      end

      def push_igetui_with_asnyc(user_device_id, template_data = {}, opts = nil)
        return if template_data.blank?

        opts ||= HashWithIndifferentAccess.new(template_type: 'TransmissionTemplate')

        template_data = HashWithIndifferentAccess.new(template_data)
        perform(template_data, opts, user_device_id)
      end

      def push_list_igetui_with_asnyc(user_device_ids, template_data = {}, opts = { pusher_type: 'push_message_to_list' })
        return if user_device_ids.blank? || template_data.blank?

        opts ||= HashWithIndifferentAccess.new(template_type: 'TransmissionTemplate')

        perform(template_data, opts, user_device_ids)
      end

      def perform(template_data, opts, user_device_ids)
        AppPush::Log.info("template_data: #{template_data}")
        AppPush::Log.info("opts: #{opts}")
        AppPush::Log.info("user_device_ids: #{user_device_ids}")
        opts = HashWithIndifferentAccess.new(opts)
        igetui = Igetui::Push.new(template_data, opts)
        result = igetui.push_message_to(user_device_ids)
        AppPush::Log.info("result: #{result}")
      end
    end
  end
end