module AppPush
  module Igetui
    class Push
      class << self
        attr_accessor :pusher, :config

        def pusher
          @pusher ||= IGeTui.pusher(
            config[:app_id],
            config[:app_key],
            config[:master_secret]
          )
        end

        def config
          (@config || default_config).with_indifferent_access
        end

        def default_config
          {
            app_id: '',
            app_key: '',
            master_secret: '',
            template_base: {
              logo: 'ik_logo.png',
              logo_url: 'http://www.ikcrm.com/assets/img/ik_logo.png',
              title: '',
              text: ''
            }
          }
        end
      end

      attr_accessor :pusher_type, :template_type, :template, :template_data

      PUSHERTYPES = %w[push_message_to_single push_message_to_list push_message_to_app].freeze
      TEMPLATETYPES = %w[LinkTemplate NotificationTemplate TransmissionTemplate NotyPopLoadTemplate].freeze

      delegate :pusher, to: 'self.class'
      delegate :config, to: 'self.class'

      def initialize(template_data, opts = {})
        opts = HashWithIndifferentAccess.new(opts)
        @template_data = HashWithIndifferentAccess.new(template_data || {})

        @pusher_type = PUSHERTYPES.include?((opts[:pusher_type]).to_s) ? opts[:pusher_type] : 'push_message_to_single'
        @template_type = TEMPLATETYPES.include?((opts[:template_type]).to_s) ? opts[:template_type] : 'NotificationTemplate'

        set_message
        set_template
        set_template_data
        set_message_data
      end

      def push_message_to(client_id)
        client_id_list = Array(client_id)
        case pusher_type
        when 'push_message_to_single'
          client_id_list.map do |client_id|
            client = IGeTui::Client.new(client_id)
            pusher.push_message_to_single(@message, client)
          end
        when 'push_message_to_list'
          content_id = pusher.get_content_id(@message)
          clients = client_id_list.map { |client_id| IGeTui::Client.new(client_id) }

          pusher.push_message_to_list(content_id, clients)
        when 'push_message_to_app'
          @message.app_id_list = client_id_list

          pusher.push_message_to_app(@message)
        end
      end

      private

      def set_template
        @template = "IGeTui::#{template_type}".constantize.new
      end

      def set_message
        @message = case pusher_type
                   when 'push_message_to_single'
                     IGeTui::SingleMessage.new
                   when 'push_message_to_list'
                     IGeTui::ListMessage.new
                   when 'push_message_to_app'
                     IGeTui::AppMessage.new
        end
      end

      def set_template_data
        case template_type
        when 'NotificationTemplate', 'NotyPopLoadTemplate', 'LinkTemplate'
          @template_data.reverse_merge!(config[:template_base])

          @template_data.each do |k, v|
            @template.send("#{k}=", v)
          end
          @template.set_push_info(*@template_data[:push_info]) if @template_data.key? :push_info
        when 'TransmissionTemplate'
          @template.transmission_content = @template_data[:transmission_content].is_a?(Hash) ? @template_data[:transmission_content].to_json : @template_data[:transmission_content]
          @template.transmission_type = @template_data[:transmission_type] if @template_data.key?(:transmission_type)
          @template.set_push_info(*@template_data[:push_info]) if @template_data.key? :push_info
        end
      end

      def set_message_data
        @message.data = @template
      end
    end
  end
end

###### useage ##############
# igetui = AppPush::Igetui.new({title: "共产党万岁", text: "中国人民大团结万岁"})
# igetui.push_message_to("ec7ef0d56cf9fa3acddacaf7730b76fd")  # client_id should bundle to app_id
#
############################
